# facebug_lib.pm is generated from facebug_lib.1.pm
# 02.01.2016
#
# Here are some functions used both by the bot and the interface
#
#    Copyright (C) 2015-2016  Balthasar Szczepański
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package facebug_lib;

use strict;
#use warnings
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

$VERSION     = 0.000000;
@ISA         = qw(Exporter);
@EXPORT      = ();
@EXPORT_OK   = qw(gettimenumber key readdatafile writedatafile);
%EXPORT_TAGS = ();

# Function to generate a random hexadecimal number (key) with a defined number
# of bits.
sub key {
	(my $bits) = @_;
	my $p = int($bits / 16);
	my $q = $bits % 16;
	my $keytext;
	
	if ($q){
	 my $r = int(($q+3)/4);
	 $keytext=sprintf('%0'.$r.'x',int(rand(1<<$q)));
	}
	for(my $ind = 0; $ind < $p; ++$ind){
		$keytext.=sprintf('%04x',int(rand(65536)));
	}
	return $keytext;
}

# Function to read data from datafiles.
# Very similar to http header file reading. (function readheaderfile() in proxy
# library)
# 
# Differences:
#
# 1. After field name and colon there must be exactly one whitespace (space or
# tab). Any other leading or trailing whitespace (but not the newline character
# at the end of the line) is treated as part of the field value.
#
# 2. When header field is split into multiple lines the next lines must start
# with exactly one whitespace (tab or space) Any other leading or trailing
# whitespace (but not the newline character at the end of the line) is treated
# as part of the field value. the lines will be joined with a newline between
# them.
#
# 3. When the same field name appears it replaces the previous one.
# 
# 4. Line separator is LF and not CR LF. The CR character is treated as part of
# the field value.
#
# Returns a hash containing the values.
# Names are case sensitive and are converted to lowercase
#
# Argument can be a path or a file handle. In case of a file handle it will just
# read the file. In case of path it opens the file before reading and closes
# after. On failure (file not open) returns empty hash.
# 
sub readdatafile {
	(my $datapath) = @_;
	my $datafile;
	my %data;
	
	# check if $datapath is actually a path or maybe a filehandle
	# filehandles are references.
	if(ref($datapath)) {
		$datafile=$datapath;
	}
	else {
		unless (open ($datafile, "<", $datapath)) {
			return %data;
		}
	}

	# The name of header field in previous line. Required for header fields that
	# occupy multiple lines.
	my $lastname='';
	
	while (defined(my $line = <$datafile>)) {
		$line =~ s/[\n]$//g;
		my $name='';
		my $value='';
		
		# Line starts with whitespace. It's a continuation of the previous line.
		# Concatenate the field value, separated by newline.
		if($line =~ /^[ \t](.*)$/){
			if($lastname ne '') {
				$data{$lastname}.="\n".$1;
			}
		}
		# Line starts with a name followed by colon. Save the value
		elsif ($line =~ /^([^:]+):[ \t](.*)$/) {
			$name = lc($1);
			$value = $2;
			
			$data{$name}=$value;
			
			$lastname = $name;
		}
	}
	
	# If argument was a path the file must be closed. 
	unless (ref($datapath)) {
		close ($datafile);
	}
	
	return %data;
}

# the function to write data to datafiles (see readdatafile() description)
#
# First argument can be a path or a file handle. In case of a file handle it
# will just read the file. In case of path it opens the file before writing and
# closes after.
#
# On failure (file not open) returns 0.
# On success returns 1.
#
sub writedatafile {
	(my $headerpath, my %header) = @_;
	my $headerfile;
	
	if(ref($headerpath)) {
		$headerfile=$headerpath;
	}
	else {
		unless (open ($headerfile, ">", $headerpath)) {
			return 0;
		}
	}
	
	foreach my $ind (keys %header) {
		my $headname = $ind;
		my $headval = $header{$ind};
		$headval =~ s/\r//g;
		$headval =~ s/\n/\n /g;
		print $headerfile "$headname: $headval\n";
	}
	print $headerfile "\n";
	
	unless (ref($headerpath)) {
		close ($headerfile);
	}
	
	return 1;
}

# Function to get a timenumber from a "date" http header field value.
# It's a 14 digit number: 4 - year, 2 - month, 2 - day, 2 - hour, 2 - minute,
# 2 - second.
# Returns undef on failure.
sub gettimenumber {
	(my $date) = @_;
	my $year;
	my $month;
	my $day;
	my $hour;
	my $minute;
	my $second;
	
	# There are 3 possible formats.
	# See https://tools.ietf.org/html/rfc2616#section-3.3.1
	if ($date =~ /^[A-Za-z]{3}, ([0-9]{2}) ([A-Za-z]{3}) ([0-9]{4}) ([0-9]{2}):([0-9]{2}):([0-9]{2})/){
		$day=$1;
		$month=lc($2);
		$year=$3;
		$hour=$4;
		$minute=$5;
		$second=$6;
	}
	elsif ($date =~ /^[A-Za-z]{3,}, ([0-9]{2})-([A-Za-z]{3})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})/) {
		$day=$1;
		$month=lc($2);
		$year='20'.$3; # Assuming 21st century!
		$hour=$4;
		$minute=$5;
		$second=$6;
	}
	elsif ($date =~ /^[A-Za-z]{3} ([A-Za-z]{3}) ([ 0-9][0-9]) ([0-9]{2}):([0-9]{2}):([0-9]{2}) ([0-9]{4})/) {
		$month=lc($1);
		$day=$2;
		$hour=$3;
		$minute=$4;
		$second=$5;
		$year=$6;
		$day =~ s/ /0/;
	}
	else {
		return undef;
	}
	
	if ($month =~ /^jan/) {
		$month = '01';
	}
	elsif ($month =~ /^feb/) {
		$month = '02';
	}
	elsif ($month =~ /^mar/) {
		$month = '03';
	}
	elsif ($month =~ /^apr/) {
		$month = '04';
	}
	elsif ($month =~ /^may/) {
		$month = '05';
	}
	elsif ($month =~ /^jun/) {
		$month = '06';
	}
	elsif ($month =~ /^jul/) {
		$month = '07';
	}
	elsif ($month =~ /^aug/) {
		$month = '08';
	}
	elsif ($month =~ /^sep/) {
		$month = '09';
	}
	elsif ($month =~ /^oct/) {
		$month = '10';
	}
	elsif ($month =~ /^nov/) {
		$month = '11';
	}
	elsif ($month =~ /^dec/) {
		$month = '12';
	}
	else {
		return undef;
	}
	
	return $year.$month.$day.$hour.$minute.$second;
}

1;
